"""Test cases for the green energy api module"""
from unittest import mock
import pytest

from green_energy_api.green_energy_api import GreenEnergyApi

ENERGY_PRICE_RESPONSE_DE = {
    "object": "list",
    "data": [
        {
            "start_timestamp": 1673942400000,
            "end_timestamp": 1673946000000,
            "marketprice": 136.1,
            "unit": "Eur/MWh"
        },
        {
            "start_timestamp": 1673946000000,
            "end_timestamp": 1673949600000,
            "marketprice": 126.17,
            "unit": "Eur/MWh"
        },
        {
            "start_timestamp": 1673949600000,
            "end_timestamp": 1673953200000,
            "marketprice": 122.75,
            "unit": "Eur/MWh"
        },
    ],
}


ENERGY_DATA_API_URL = "https://api.awattar.de"


# pylint: disable=unused-argument
def mocked_requests_get(*args, **kwargs):
    """Module handling mocked API requests"""

    # pylint: disable=too-few-public-methods
    class MockResponse:
        """Class handling mocked API responses"""

        def __init__(self, json_data, status_code):
            self.json_data = json_data
            self.status_code = status_code

        def json(self):
            """Return data as a JSON"""
            return self.json_data

    if args[0] == f"{ENERGY_DATA_API_URL}/v1/marketdata":
        return MockResponse(ENERGY_PRICE_RESPONSE_DE, 200)

    return MockResponse(None, 404)


@mock.patch(
    "requests.get",
    mock.Mock(side_effect=mocked_requests_get),
)
def test_get_co2_forecast() -> None:
    """Test if the forecast data is returned"""
    api = GreenEnergyApi(ENERGY_DATA_API_URL)
    assert api.get_electricity_price() == ENERGY_PRICE_RESPONSE_DE


@mock.patch(
    "requests.get",
    mock.Mock(side_effect=mocked_requests_get),
)
def test_request_status_error() -> None:
    """Test if API call fails"""
    api = GreenEnergyApi("http://localhost:3001")
    with pytest.raises(Exception) as exception:
        api.get_electricity_price()
    assert str(exception.value) == "Request failed with: None"



"""Package setup"""

from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="green_energy_api",
    version="0.2.0",
    author="",
    author_email="",
    description="A Python API for receiving energy prices from the Awattar API",
    license="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    provides=["green_energy_api"],
    install_requires=["requests"],
    setup_requires=["wheel"],
)



"""
Awattar Energy Price module, documentation:
https://www.awattar.de/services/api
"""


import requests

from .validations import validate_empty_string


# pylint: disable=too-few-public-methods
class GreenEnergyApi:
    """
    Class providing methods for getting CO2 forecast for a zone.
    """

    def __init__(self, host: str, timeout: int = 5) -> None:
        validate_empty_string(host, "host")
        self.host: str = host
        self.timeout: int = timeout

    def __query_avattar_api(self, path: str) -> dict:
        try:
            url = f"{self.host}/{path}"

            status_request = requests.get(url, timeout=self.timeout)
            status = status_request.json()
            return status
        except (
            requests.exceptions.ConnectTimeout,
            requests.exceptions.ConnectionError,
        ):
            return {"success": False, "msg": "Request couldn't connect or timed out"}

    def get_electricity_price(self) -> dict:
        """Get the energy prices from a public api"""
        response = self.__query_avattar_api("v1/marketdata")

        if response is None or response.get("success") is False:
            raise RuntimeError(f"Request failed with: {response}")

        return response
